# distutils based setup script
# for making tarballs to ease distribution

from distutils.core import setup

setup(name="moosic-applet",
      version="0.0.4",
      description="Gnome applet for controlling Moosic",
      author="Paramjit Oberoi and Daniel Pearson",
      author_email="param@cs.wisc.edu",
      url="http://www.nanoo.org/~daniel/moosic/",
      license="GPL",
     )
